/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 /*
 * Author: Paul Beaulieu
 *
 * Summary:
 *   The principle used here is from a web forum where a question about measuring strings in html was
 *   posed.
 *   
 *   This utility uses the principle that the text set to a DOM node will have a particular width
 *   as a result of its font properties.  This utility given the container node where the string
 *   will be displayed, the node used as a ruler for measuring the string in pixels, and 
 *   a string will put hyphens and spaces in the string so tha the applied CSS can wrap the string
 *   appropriately.  
 *   
 *   In order for this to work, the ruler node needs to be visible.  It can be off screen,  but a display:none
 *   string will not work because the DOM will not consider it part of the display.  Furthermore, the ruler node
 *   have the same CSS font styles applied to it as the string would have applied when it is placed in the container
 *   node.  Otherwise, the measurements could be considerable off. 
 *   
 */
dojo.provide("com.ibm.asq.common.web.ui.internal.util.wordWrapping");
dojo.provide("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");

(function() {

// Summary:
//    "Static" function to insert "word breaks" into a single word in a text string.
//    In situations where a long, unbreakable word (in Western languages), we want
//    the word to wrap (goto next line) to fit into a fixed width area. This routine
//    will break the words in a text string that exceeds the width of the prescribed
//    width. The word will be padded with a '-' and a word break string, like a space,
//    or an HTML tag '<br/>'. Once the text string is processed, the string will be
//    able to fit inside the given width.
//
//    Note: client can use this method directly, or use the mixin class defined below.
//    The mixin class was written first. This new method was refactored out to be a
//    standalone method and doesn't require the client to add the mixin class.
//
com.ibm.asq.common.web.ui.internal.util.wordWrapping.getWordWrappedString =
function( maxWidth, rulerNode, string, wordBreakStr )
{
    //Get the area that is displaying the title
    var titleWidth = maxWidth;
    var newString = "";
    var numChars = 0;
    
    // Check for reasonable parameters before doing things.
    if(titleWidth == 0 || !string || string.length == 0 || rulerNode == null) {
        return string;
    }
    
    //Split the strings on spaces
    var strs = string.split( " " );
    
    // if the workbreak string/char is a space, we've to count
    // it too. 
    var offset = (wordBreakStr == " ") ? 2 : 1;
    
    //Loop through each string in the split array and find strings that are longer in pixels 
    //than the width of the area that can contain them.
    for( var iString = 0; iString < strs.length; iString++ )
    {
        //Assign the string to the ruler
        rulerNode.innerHTML = strs[ iString ];
        //Get the pixel width of the string from the ruler
        var width = rulerNode.offsetWidth;
        
        if( width > titleWidth )
        {
            //if the string would not fit in the container then proceed to cut it up
            if( iString != 0 )
                newString += wordBreakStr;
                
            //Find the longest amount of the current string and that can fit in the container.
            for( var index = strs[ iString ].length - 1; index >= 0; index-- )
            {
                var str = strs[ iString ].substring( 0, index );
                rulerNode.innerHTML = str;
                if( rulerNode.offsetWidth < titleWidth )
                {
                    numChars = index;
                    break;
                }
            }
            
            // degenerate to break up all of the chars if we can't even fit one char.
            // Display wise, it probably doesn't make sense and won't look good
            // at this point. But at least the code wouldn't break and gets into
            // divid by 0, etc.
            // 
            numChars = Math.max(numChars, 1);
            
            //Take the current string that is too long and cut it up into pieces that can be wrapped
            //by the CSS rules.
            var start = 0;
            var end = Math.max(1, (numChars - offset));//Includes the space for the "- ", that is why it is less 2 more than it should be.
            
            for( var index = 0; index < ( strs[ iString ].length ) / numChars; index++ )
            {                       
                if( start == 0 )
                    newString += strs[ iString ].slice( start, end );
                else
                    newString += "-" + wordBreakStr + strs[ iString ].slice( start, end );
                    
                start = end;
                
                if( ( index + 1 ) < ( strs[ iString ].length ) / numChars )
                    end = end + Math.max(1, (numChars - offset));//Includes the space for the "- ", that is why it is less 2 more than it should be.
                else
                    end = end + numChars;
            }
        }
        else
        {
            //If the string is under the length of the container then proceed here
            if( newString.length > 0 )
                newString += wordBreakStr;
                
            newString += strs[ iString ];
        }
    }
    
    return( newString );
};

// Summary:
//    Given a string and a maximum width, this routine wraps the string
//    with the given line break (usually the HTML tag <br/>) if the string
//    exceeds the width. This routine will only attempt to wrap the text
//    at spaces. However, if the parameter 'breakWord' is set to true, words
//    within the string will be broken up if it cannot be fitted in the
//    prescribed width.
//
com.ibm.asq.common.web.ui.internal.util.wordWrapping.wrapText = 
function (string, className, maxWidth, breakWord, lineBreakStr)
{    
    if(!string || string.length == 0 || maxWidth == 0 || !lineBreakStr || lineBreakStr.length == 0)
        return string;
    
    var ruler = document.createElement("span");
    
    // create and setup the temporary ruler.
    ruler.className = className;
    ruler.style.position = "absolute";
    ruler.style.left = "-3000px";
    ruler.innerHTML = string;
    document.body.appendChild(ruler);
   
    // simple case: if the string fits, don't have
    // to do anything.
    if(ruler.offsetWidth <= maxWidth) {
         document.body.removeChild(ruler);
         return string;
    }
    
    var newString = "";
    var aLine = "";
    
    //Split the strings on spaces
    var words = string.split(" ");
    for(var i = 0; i < words.length; i++) {
        aLine += words[i];
        ruler.innerHTML = aLine;
        
        // if a word is wider then the width and we are
        // breaking the word too, go ahead and break up
        // the word.
        if(breakWord && ruler.offsetWidth > maxWidth) {
            aLine = com.ibm.asq.common.web.ui.internal.util.wordWrapping.getWordWrappedString(maxWidth, ruler, words[i], lineBreakStr);
        }
        
        // try to fit as many words into a line until we have to wrap.
        var j = i+1;
        while (j < words.length) {
            var temp = aLine + " " + words[j];
            ruler.innerHTML = temp;
            
            // the 'next' word put us over the max width;
            // don't take it and insert the line break.
            if(ruler.offsetWidth > maxWidth) {
                break;
            }
            aLine = temp;
            i = j;
            j++;
        }
        // the string can't fit, insert a line break and start another new line.
        if (i < words.length) {
            aLine += lineBreakStr;
        }
        newString += aLine;
        aLine = "";
    }
    
    document.body.removeChild(ruler);        
    return newString;
};

var Slack = 4;
var Ellipsis = "...";

// Summary:
//    Given a string, this routine clips the text at the specified
//    max width and appends an ellipsis at the end of text if the
//    text string was clipped and the 'showEllipsis' parameter is true.
//
//    The method returns an object : {clipped: <true|false>, text: clipped, or unclipped string}
//
com.ibm.asq.common.web.ui.internal.util.wordWrapping.clipText = 
function(string, className, maxWidth, ruler, showEllipsis, threshold)
{
    // cannot really clip a empty or 1 char string.
    if(!string || string.length < 2)
        return {'clipped': false, 'text': string};
            
    var createdRuler = false;
    
    if(!ruler) {
        ruler = document.createElement("span");
        
        // create and setup the temporary ruler.
        ruler.style.position = "absolute";
        ruler.style.left = "-3000px";
        document.body.appendChild(ruler);
        createdRuler = true;
    }

    // make sure that the threshold is not possibly smaller then
    // the width of a single char. Otehrwise, we could get into 
    // infinite loop trying to make the string fit.
    ruler.className = className;
    ruler.innerHTML = "W";
    var minCharWidth = ruler.offsetWidth + Slack;
    
    if(typeof threshold == "undefined" || threshold < minCharWidth) {
        threshold = minCharWidth;
    }

    // make sure the maxWidth is a reasonable value. If it's 
    // smaller then the threshold, we can't clip anything.   
    if (maxWidth <= threshold) {
        if(createdRuler)
            document.body.removeChild(ruler);
        return {
            'clipped': true,
            'text': (showEllipsis) ? Ellipsis : ""
        };
    }
    ruler.innerHTML = string;
    
    // simple case: if the string fits, don't have
    // to do anything.
    if(ruler.offsetWidth < maxWidth) {
        if(createdRuler)
            document.body.removeChild(ruler);
        return {'clipped': false, 'text': string};
    }

    var justFit = false;
    var endPos = Math.round(Math.max(string.length, 2) / 2);
    var nextHalf = endPos;
    var partial;
    var ellipsisWidth = 0;
    var changedDir = 0;
    var lastDir = 1;
    
    if(showEllipsis) {
        ruler.innerHTML = Ellipsis;
        ellipsisWidth = ruler.offsetWidth;
    }
    
    while(!justFit) {
        partial = string.substr(0, endPos);
        ruler.innerHTML = partial;
        
        var curWidth = ruler.offsetWidth + ellipsisWidth;
        var diff = maxWidth - curWidth;
        
        // the string width is less than the max width and within 
        // the acceptable threshold, the string is good. 
        if (diff >= 0 && diff <= threshold) {
            justFit = true;
        }
        else {
            var dir = (diff > 0) ? 1 : -1;
            nextHalf = (nextHalf == 1) ? nextHalf : Math.round(nextHalf / 2);
            endPos += ((diff > 0) ? nextHalf : (0 - nextHalf));
            
            // if we keep going back and forth on a single char, the 
            // threshold for the fit is too narrow. (this shouldn't happen)
            // We cannot split smaller then a single char, just leave
            // at this point.
            if(nextHalf == 1) { 
                if (lastDir != dir) {
                    changedDir++;
                    if (changedDir > 10) 
                        break;
                }
            }
            lastDir = dir;
        }
    }
    return {'clipped': true, 'text':(showEllipsis) ? partial + Ellipsis : partial};
};

dojo.declare("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping",
null,
{	
	getWordWrappedString: function( stringContainerNode, rulerNode, string )
	{
		//Get the area that is displaying the title
		var titleWidth = (stringContainerNode) ? stringContainerNode.offsetWidth : 0;
        var wordBreak = " ";
        return com.ibm.asq.common.web.ui.internal.util.wordWrapping.getWordWrappedString(titleWidth, rulerNode, string, wordBreak);
	}
});

})();